/*-----------------------------------------------------------------------------
 *Projeto - taco_bike AT89C2051 - tacômetro para bicicletas!!!!
 * 
 * Desenvolvido por.: Eng. Márcio José Soares
 * 
 * microcontrolador - AT89C2051, clock a 12MHz
 * Compilador       - sdcc 3.8.0 (Linux)
 * 
 * Gravador: minipro
 * 			- make compile (p/ compilar apenas)
 * 			- make program (p/ gravar após compilar)
 * 			- make all (p/ compilar e em seguida gravar)
 * 
 * Pinos:
 * 
 * PORT1
 * P1.0 - . (display 7 segmentos)     ___A___
 * P1.1 - C (display 7 segmentos)    |       |
 * P1.2 - D (display 7 segmentos)   F|       |B
 * P1.3 - E (display 7 segmentos)    |       |
 * P1.4 - B (display 7 segmentos)     ___G___
 * P1.5 - A (display 7 segmentos)    |       |
 * P1.6 - F (display 7 segmentos)   E|       |C
 * P1.7 - G (display 7 segmentos)    |       |
 *                                    ___D___
 * 
 * PORT3
 * P3.0 - BTN PAUSA
 * P3.1 - n.c.
 * P3.2 - SENSOR
 * P3.3 - BTN MUDA
 * P3.4 - COL0 - DISP3 (displays 7 segmentos)
 * P3.5 - COL1 - DISP2 
 * P3.7 - COL2 - DISP1
 * 
-----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 * Arquivos incluídos no módulo
-----------------------------------------------------------------------------*/
#include <mcs51/at89x051.h>
#include <stdint.h>
#include <stdio.h>

/*-----------------------------------------------------------------------------
 * Definições importantes do módulo
-----------------------------------------------------------------------------*/
#define DISP 	P1				/* porta dos displays */
#define MUDA    P3_3
#define PAUSA	P3_0

/*  Posição para os pinos conforme esquema e montagem no pront-o-board	
    #define COL0 P3_4 unidade 
	#define COL1 P3_5 dezena 
	#define COL2 P3_7 centena */

/* Posição dos pinos conforme placa! Troca de pinos para faciliara layout*/
#define COL0	P3_7		
#define COL1	P3_4		
#define COL2 	P3_5		

#define TRUE	1				/* */
#define FALSE	0

#define TIMET0	100 			/* timer tem uma int a cada xms */

/*-----------------------------------------------------------------------------
 * Variáveis globais
-----------------------------------------------------------------------------*/
uint8_t unidade = 0;
uint8_t dezena = 0;
uint8_t centena = 0;
uint8_t ponto_unidade = 0;
uint8_t ponto_dezena = 0;
uint8_t ponto_centena = 0;

uint8_t time_1s = 0;
uint8_t time_1m = 0;
uint8_t pos = 0;
uint8_t pausa = 0;
uint8_t tmp;

uint8_t time10s = 0;
uint8_t parado = 0;

volatile uint16_t timer_counter = 0; 
volatile uint16_t counter_cpy = 0;
volatile uint16_t dist = 0;
volatile uint16_t tempo = 0;

volatile uint32_t vel = 0;
volatile uint32_t aux = 0;
volatile uint8_t showpos = 0;
volatile uint8_t pisca = 0;
volatile uint8_t t_pisca = 0;
volatile uint8_t qt_pisca = 0;

char _7seg[14] = {0x81,0xED,0x43,0x49,0x2D,0x19,0x11,0xCD,0x01,0x0D,0x07,0x20,0x80,0x04};

/*-----------------------------------------------------------------------------
 * Funções do módulo
-----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
 Função delay - espera ocupada em assembly 8051
 
 Usa timer 1 para essa contagem! ok em 12MHz!!!
 
 Função aguarda 1000us x tempo em ms
 
 Entradas - byte quantidade de ms a aguardar
 Saídas   - nenhuma
-----------------------------------------------------------------------------*/
void delay(unsigned char ms){
	
    tmp = ms;
	
__asm

dl0: mov 	th1,#0x0FC	; carrega registrador TH com 64536
	 mov 	tl1,#0x018
	 setb 	tr1			; inicia timer 1
dl1: jnb 	tf1,dl1		; laço até que flag TF1 seja setado
     clr 	tr1			; para timer 1
     clr 	tf1			; limpa flag
     djnz	_tmp,dl0	; decrementa ms e continua até zerar!
    
__endasm;
	

}

/*-----------------------------------------------------------------------------
 Função para distribuir valor em counter_cpy no display nas variáveis:
 centena, dezena e unidade

 Entradas - nenhuma
 Saídas   - nenhuma
-----------------------------------------------------------------------------*/
void arruma_vars(void){
	
	if(!pausa){
		if(counter_cpy >= 999){
			counter_cpy = 0;
			centena = 0;
			dezena = 0;
			unidade = 0;
		}
		else{
			if(counter_cpy >= 100){									/* se tem centena */
				centena = (uint8_t)(counter_cpy / 100);
				dezena = (uint8_t)((counter_cpy % 100) / 10);
				unidade = (uint8_t)(((counter_cpy % 100) % 10));
			}
			else{
				centena = 0;										/* do contrário centena ainda é zero */
				if(counter_cpy >= 10){								/* se term dezena */
					dezena = (uint8_t)(counter_cpy / 10);
					unidade = (uint8_t)(counter_cpy % 10);
				}
				else{
					dezena = 0;
					unidade = counter_cpy;
				}
			}
		}
	}
	else{
		centena = 0x0A;
		dezena = 0x0A;
		unidade = 0x0A;
		ponto_centena = 0x00;
		ponto_dezena = 0x00;
		ponto_unidade = 0x00;
	}
}

/*-----------------------------------------------------------------------------
 Função da int externa 0 - sensor

 Entradas - nenhuma
 Saídas   - nenhuma
-----------------------------------------------------------------------------*/
void int0_isr(void) __interrupt (0)	{
	
	if(!pausa){
		vel = (uint32_t)(1570 / timer_counter);		/* vel em ms */
		if(vel > 20){								/* permite mostrar velocidades */
			vel = vel/10;							/* altas e baxas!!! */
			vel = vel*360;
		}
		else{
			vel = vel * 36;
		}
			
		dist += 1;									/* soma distância */
		parado = 1;
		
		switch(pos){								
			case 0:									/* é para mostrar a velocidade */
				if(vel >= 100 && vel < 1000){
					ponto_centena = 0x01;
					ponto_dezena = 0x00;
					ponto_unidade = 0x00;
				}
				else if(vel >= 1000){
					vel = vel / 10;
					ponto_centena = 0x00;
					ponto_dezena = 0x01;
					ponto_unidade = 0x00;
				}
				else{
					ponto_centena = 0x00;
					ponto_dezena = 0x00;
					ponto_unidade = 0x00;
				}
				counter_cpy = (uint16_t)vel;		/* copia */
				break;
			case 1:									/* é para mostrar distância percorrida */
				aux = (uint32_t)((dist * 15)/10);	/* copia */
				if(aux >= 100 && aux < 1000){
					ponto_centena = 0x00;
					ponto_dezena = 0x01;
					ponto_unidade = 0x00;
					aux = aux / 100;
				}
				else if(aux >= 1000){
					ponto_centena = 0x00;
					ponto_dezena = 0x01;
					ponto_unidade = 0x00;
					aux = aux / 100;
				}
				else{
					ponto_centena = 0x00;
					ponto_dezena = 0x00;
					ponto_unidade = 0x00;
					aux = 0;
				}
				counter_cpy = (uint16_t)(aux);
				break;
			case 2:
				counter_cpy = (uint16_t)tempo;
				ponto_centena = 0x00;
				ponto_dezena = 0x00;
				ponto_unidade = 0x00;
				break;
			default:
				counter_cpy = 0;
				ponto_centena = 0x00;
				ponto_dezena = 0x00;
				ponto_unidade = 0x00;
				
		}
			
		timer_counter = 0;							/* zera */
		vel = 0;
		aux = 0;
	}
}

/*-----------------------------------------------------------------------------
 Função da int do timer0 

 65536 - 10536 = 50000 x 1us = 50ms (0x2928 -> 10536d)
 
 Entradas - nenhuma
 Saídas   - nenhuma
-----------------------------------------------------------------------------*/
void timer0_isr(void) __interrupt (1) {

	TR0 = 0;										/*para o timer*/
	
	if(!pausa){
		timer_counter++;
		time_1s++;									/* incrementa até chegar a 1 segundo */
		if(time_1s >= 100){							/* 10ms x 100 = 1000ms */
			time_1s = 0;
			
			if(showpos){							/* se está fazendo a troca da operação */
				t_pisca = 1;
				timer_counter = 0;
			}
			
			time_1m++;								/* incrementa até chegar a 1 minuto */
			if(time_1m >= 60){			
				time_1m = 0;
				tempo++;							/* incrementa a var tempo */
			}
			
			time10s++;
			if(time10s >= 10){						/* passou 10s ?? */
				time10s = 0;
				if(parado == 0){					/* se parado após 10 segundos */
					counter_cpy = 0;				/* mostra zero nos displays */
					timer_counter = 0;				/* zera contador */
					vel = 0;						/* zera vel */
					ponto_centena = 0x01;			/* liga todos os pontos */
					ponto_dezena = 0x01;
					ponto_unidade = 0x01;
				}
				else
					parado = 0;						/* limpa parado */
			}
		}
	}
	
    
    TH0 = 0xD8;   									/* restabelece valor inicial para contador 0*/
	TL0 = 0xF0; 
    TF0 = 0;										/* limpa flag da int */
	TR0 = 1;										/* inicia timer novamente */
}


/*-----------------------------------------------------------------------------
 Função main

 Entradas - nenhuma
 Saídas   - nenhuma
-----------------------------------------------------------------------------*/
void main(void){
    
	uint8_t coluna = 0;
	int i = 0;
	
	/* Configura timers */
	PCON  = 0x00;		/* desliga comparadores */
    TMOD &= 0x00;		/* Timer1 usado na função delay, limpa timer 0 */
    TMOD |= 0x11; 		/* Timer0 em mode 1: 16 bit timer */
						/* Timer1 em mode 1: 16 bit timer */
						
    TH0 = 0xD8;   		/* valor inicial para contador 0 - 65536 - 10536 (2928H) = 50000 x 1us = 50ms */
    TL0 = 0xF0;   		/* valor inicial para contador 0 - 65536 - 40536 (9E58H) = 25000 x 1us = 25ms */
						/* valor inicial para contador 0 - 65536 - 55536 (D8F0H) = 10000 x 1us = 10ms */
    TH1 = 0x00;			/* limpa contador 1*/
	TL1 = 0x00;
	
    ET0=1;        		/* Habilita int do timer0 */
    ET1=0;				/* Desabilita int do timer 1 */
    
    TR0=1;        		/* liga timer 0 */
    TR1=0;				/* mantem timer 1 desligado */

    /* Configura pinos de I/O */
    P1 = 0x00;			/* port P1 como saída */
	P3 = 0x0F;			/* port P3 lsb entrada, msb saída */
	
	/* Configura INTs externas */
	TCON |= 0x01;		/* INT0 ativa na borda de descida */
	PX0 = 1;			/* INT0 tem mais prioridade que INT1 */
	PX1 = 0;
	EX0 = 1;			/* Habilita INT0 */
	EX1 = 0;			/* não habilita INT1 */
	
	EA=1;        		/* Habilita int global */
	
	P3_4 = 1;			/* liga coluna 0, desliga demais */
	P3_5 = 1;
	P3_7 = 1;
	
    while(TRUE) {
		if(!showpos)				/* se não precisa mostrar a posição */
			arruma_vars();			/* arruma vars para poder mostrar nos displays */
		else{						/* precisa mostrar os traços */
			if(pisca){
				switch(pos){			/* seleciona o que vai mostar */
					case 0:
						unidade = 0x0B;	/* todos segmentos A acesos */
						break;
					case 1:
						unidade = 0x0C;	/* segmentos G acesos */
						break;
					case 2:
						centena = 0x0D; /* segmentos D acesos */
						break;
					default:
						unidade = 0x0B;
				}
			}
			else{
				unidade = 0x0E;
			}
			dezena = unidade;
			centena = unidade;
			if(t_pisca == 1){			/* se já deu o tempo */
				t_pisca = 0;			/* prepara para trocar */
				pisca ^= 0x01;
				qt_pisca++;				/* faz 6 vezes */
				if(qt_pisca >= 6){
					showpos = 0;		/* sai de mostra troca */
					qt_pisca = 0;		/* zera quantidade de piscadas */
					pisca = 0;			/* inicia apagado */
					timer_counter = 0;
					EX0 = 1;			/* Habilita novamente a INT0 */
					continue;			/* volta para o inicio do loop */
				}
			}
		}
		
		switch(coluna){
			case 0:
				COL0 = 0;			 /* liga coluna 0, desliga demais */
				COL1 = 1;
				COL2 = 1;
				DISP = _7seg[unidade] & ~ponto_unidade;
				coluna = 1;
				break;
			case 1:
				COL0 = 1;			 /* liga coluna 1, desliga demais */
				COL1 = 0;
				COL2 = 1;
				DISP = _7seg[dezena] & ~ponto_dezena;
				coluna = 2;
				break;
			case 2:
				COL0 = 1;			 /* liga coluna 2, desliga demais */
				COL1 = 1;
				COL2 = 0;
				DISP = _7seg[centena] & ~ponto_centena;
				coluna = 0;
				break;
			default:
				;
		}
		delay(5);					/* aguarda 30 ms segundos */
		
		if(!PAUSA){					/* lê chave pausa */
			pausa = 0x01;			/* tempo em pausa */
		}
		else{
			pausa = 0x00;			/* tempo em pausa */
		}
		
		if(!MUDA){					/* pediu para mudar informação no display */
			while(!MUDA)			/* enquanto não solta a chave */	
				for(i=0;i<1000;i++) /* segura em espera ocupada */
					;
			pisca = 1;				/* inicia ligado */
			qt_pisca = 0;			/* zera quantidade de piscadas */
			t_pisca = 0;			/* zera o tempo de 1 segundo on/off */
			EX0 = 0;				/* Desabilita INT0 */	
			pos++;					/* soltou a chave, incrementa... */
			if(pos >= 3)			/* se passou do máximo, volta ao inicio... */
				pos = 0;
			
			showpos = 1;			/* avisa que botão foi pressionado */
		}
		
	}
    
}
